/* *****************************************************************************
 * libg1m/format/mcs/matrix.h -- description of the MCS matrix format.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#ifndef LIBG1M_FORMAT_MCS_MATRIX_H
# define LIBG1M_FORMAT_MCS_MATRIX_H
# include <libg1m/bcd.h>
# pragma pack(1)

/* Matrixes and vectors have exactly the same format; it's just that the OS
 * refuses to have more that one row for vectors.
 *
 * They have this simple header: */

struct mcs_matheader {
	/* undocumented */
	uint8_t undocumented[8];

	/* height */
	uint16_t height;

	/* width */
	uint16_t width;

	/* re-undocumented */
	uint8_t undocumented2[4];
};

/* Then we have width*height BCD cells, which corresponds to the real parts,
 * grouped by height.
 *
 * If at least one of the cells has an imaginary part (highest bit of the
 * first nibble set), it is followed by a list of the imaginary parts, that
 * has the same size that the real parts list, that is grouped the same way and
 * that contains actual things (and not crap) only if the complex bit is set
 * on the real part. */

# pragma pack()
#endif /* LIBG1M_FORMAT_MCS_MATRIX_H */
