/* *****************************************************************************
 * libg1m/format/mcs/spreadsheet.h -- description of the MCS spreadsheet format.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#ifndef LIBG1M_FORMAT_MCS_SPREADSHEET_H
# define LIBG1M_FORMAT_MCS_SPREADSHEET_H
# include <libg1m/bcd.h>
# pragma pack(1)

/* Spreadsheets are more complicated than that.
 *
 * For normal ones, there is a header: */

struct mcs_spreadsheetheader {
	/* has subheader: 0x01 if yes */
	uint8_t has_subheader;

	/* column count (max: 26), on 24 bits and non-aligned */
	uint32_t column_count : 24;
};

/* Then, if it is a normal spreadsheet, there is a subheader,
 * a column directory and a column definition table.
 *
 * Here's the subheader: */

struct mcs_spreadsheet_subheader {
	/* alignment or magic? {0, 0, 0, 0} */
	uint8_t align[4];

	/* number of column definitions */
	uint32_t defs_size;

	/* alignment or magic II? {0, 0, 0, 0} */
	uint8_t align2[4];
};

/* The column definition table is the main definition. It contains the row
 * directory, which is a 80-bytes long bitfield indicating if cells are empty
 * or not, and the list of non-empty cells.
 *
 * Each cell is a BCD structure (the real part of the number). Numbers in
 * spreadsheets can't have imaginary parts.
 *
 * The column directory is just a list of 4-bytes sizes of each column,
 * counting the row directory and the cells. It's easier to naviguate to
 * a column quicker. */

# pragma pack()
#endif /* LIBG1M_FORMAT_MCS_SPREADSHEET_H */
