/* *****************************************************************************
 * decode/mcs/alphamem.c -- decode an MCS alpha memory file.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_decode_mcs_alphamem:
 *	Decode alpha memory (variables).
 *
 *	@arg	handle		the handle.
 *	@arg	buffer		the buffer to read from.
 *	@arg	length		the data length.
 *	@return				the error code (0 if ok).
 */

int g1m_decode_mcs_alphamem(g1m_mcsfile_t *handle, g1m_buffer_t *buffer,
	uint_fast32_t length)
{
	/* read the data */
	uint8_t buf[length];
	READ(buf, length)

	/* count number of vars */
	handle->count = length / (2 * sizeof(mcs_bcd_t));

	/* allocate the vars table */
	handle->vars = malloc(handle->count * sizeof(g1m_mcs_cell_t));
	if (!handle->vars) return (g1m_error_alloc);

	/* copy */
	const mcs_bcd_t *b = (void*)buf;
	for (int i = 0; i < handle->count; i++) {
		g1m_bcd_frommcs(b++, &handle->vars[i].real);
		g1m_bcd_frommcs(b++, &handle->vars[i].imgn);
		handle->vars[i].used = 1;
	}

	/* copy */
	size_t tocopy = length - length % sizeof(g1m_mcs_cell_t);
	memcpy(handle->vars, buf, tocopy);

	/* no problem, woop woop */
	return (0);
}
