/* *****************************************************************************
 * utils/limbuffer.c -- limited libg1m buffer utilities.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_limbuffer_read:
 *	Read from a filebuffer.
 *
 *	@arg	vcookie		the FILE* (uncasted)
 *	@arg	buf			the buffer to fill.
 *	@arg	size		the size to read.
 *	@return				the error (if any).
 */

int g1m_limbuffer_read(void *vcookie, unsigned char *buf, size_t size)
{
	/* check if the size is okay */
	g1m_limited_t *lim = (void*)vcookie;
	if (size > lim->left) return (g1m_error_eof);
	g1m_buffer_t *buffer = lim->buffer;

	/* read */
	READ(buf, size)

	/* no error */
	return (0);
}

/**
 *	g1m_empty_limbuffer:
 *	Empty the limit buffer.
 *
 *	@arg	limbuf		the limbuffer.
 */

int g1m_empty_limbuffer(g1m_buffer_t *limbuffer)
{
	g1m_limited_t *lim = (void*)limbuffer->cookie;
	g1m_buffer_t *buffer = lim->buffer;
	SKIP(lim->left)
	return (0);
}
